#version 330 core

layout(pixel_center_integer) in vec4 gl_FragCoord; // pixel positions at integer coordinates

uniform vec3 color;
uniform uint colorID;
uniform float farPlane = 200.f;

uniform sampler2D depthMap;
uniform bool useDepthMap;

uniform vec3 viewLightPosition;

uniform vec2 invImageSize; // 1.0/width, 1.0/height

out vec4 outColor;
out uint outID;

in vec3 vf_normal;
in vec3 vf_viewPosition;
in vec3 vf_color;

void main(void)
{
	float depth = -vf_viewPosition.z / farPlane;

	// if fragment is inside the cutting volume -> discard
	if (useDepthMap)
	{
		vec2 center = (gl_FragCoord.xy + vec2(0.5,0.5));
		vec2 centerp = center * invImageSize;

		float depthMapSample = texture(depthMap, centerp).r;
		if (depth < depthMapSample)
		{
			discard;
		}
	}

	vec3 l = normalize(viewLightPosition - vf_viewPosition);
	vec3 n = normalize(vf_normal);
	float light = clamp(dot(l, n), 0.0, 1.0);

	float lightCell = floor(light*8.0) / 8.0;
	float lightSmaller = lightCell*0.5+0.4;
	
	outColor = vec4(vf_color*color*lightSmaller, 1.0);
	outID = colorID;

	gl_FragDepth = depth;
}