#version 330 core

uniform sampler2D texture0;
uniform vec2 invImageSize;
uniform vec2 threshold;

uniform sampler2D depthMap;
uniform bool useDepthMap;

in vec2 vf_texCoords;

out vec4 outColor;

void main(void)
{
	float texSample = texture(texture0, vf_texCoords).r;

	float round[8];
	round[0] = texture(texture0, vf_texCoords+vec2(invImageSize.x,invImageSize.y)).r;
	round[1] = texture(texture0, vf_texCoords+vec2(invImageSize.x,0)).r;
	round[2] = texture(texture0, vf_texCoords+vec2(invImageSize.x,-invImageSize.y)).r;
	round[3] = texture(texture0, vf_texCoords+vec2(0,-invImageSize.y)).r;
	round[4] = texture(texture0, vf_texCoords+vec2(-invImageSize.x,-invImageSize.y)).r;
	round[5] = texture(texture0, vf_texCoords+vec2(-invImageSize.x,0)).r;
	round[6] = texture(texture0, vf_texCoords+vec2(-invImageSize.x,invImageSize.y)).r;
	round[7] = texture(texture0, vf_texCoords+vec2(0,invImageSize.y)).r;

	float minDepth = 1.0;

	float maxGradDif = 0;
	for (int i=0;i<4;i++)
	{
		float r1 = round[i];
		float r2 = round[(i+4) % 8];
		minDepth = min(minDepth, r1);
		minDepth = min(minDepth, r2);

		float dif1 = r1 - texSample;
		float dif2 = texSample - r2;
		float gradDif = abs(dif1-dif2);
		maxGradDif = max(maxGradDif, gradDif);
	}

	float fadeFactor = 1.0f;

	if (useDepthMap)
	{
		vec2 center = (gl_FragCoord.xy + vec2(0.5,0.5));
		float gaussSample = texture(depthMap, vf_texCoords).r;
		fadeFactor = clamp(1.f - 6.f*gaussSample, 0.f, 1.f);
	}
	
	float color = fadeFactor * clamp((maxGradDif-threshold.x) / (threshold.y), 0.0, 1.0);

	outColor = vec4(color,color,color,1.0);
}