#include "CameraController.h"
#include <iostream>

using namespace std;

CameraController::CameraController(Camera* camera, InputHandler* inputHandler)
	: _camera(camera), _state(IDLE), _inputHandler(inputHandler), _pivot(0.f,0.f,0.f),
	_lastMouseWheel(0)
{
	_position = _camera->getPosition();
}

bool CameraController::isActive() const
{
	return _state!=IDLE;
}

void CameraController::setPivot(const glm::vec3& pivot)
{
	_pivot = pivot;
	_camera->setTarget(_pivot);
	_camera->updateTransform();
}

void CameraController::startOrbit()
{
	_state = ORBIT;
	_startPosition = _position;

	glfwGetMousePos(&_startMousePosX, &_startMousePosY);
}

void CameraController::stopOrbit()
{
	_state = IDLE;
}

void CameraController::zoomBy(int zoom)
{
	glm::vec3 pos = _position - _pivot;
	pos *= (1.f - 0.04f*(float)zoom);
	_position = pos+_pivot;

	_camera->setPosition(_position);
	_camera->updateTransform();
}

void CameraController::update(double frameLength)
{
	if (_state == ORBIT)
	{
		int newMousePosX, newMousePosY;
		glfwGetMousePos(&newMousePosX, &newMousePosY);

		int distMouseX = newMousePosX - _startMousePosX;
		int distMouseY = newMousePosY - _startMousePosY;

		// pan
		float angle = (float)distMouseX / 200.f;

		_position = _startPosition - _pivot;

		float pannedX = cos(angle)*_position.x - sin(angle)*_position.z;
		float pannedZ = sin(angle)*_position.x + cos(angle)*_position.z;

		_position.x = pannedX;
		_position.z = pannedZ;

		float rad = sqrt(_position.x*_position.x + _position.z*_position.z);

		// tilt
		float angle2 = (float)distMouseY / 200.f;

		float newRad = cos(angle2)*rad - sin(angle2)*_position.y;
		float newY = sin(angle2)*rad + cos(angle2)*_position.y;

		float critAngle = 3.1415f*0.5f*0.9f;
		float tanCritAngle = tan(critAngle);

		if (newY > 0 && (tanCritAngle < newY/newRad || newRad < 0))
		{
			float len = glm::length(_position);

			newRad = len * cos(critAngle);
			newY = len * sin(critAngle);
		}
		else if (newY < 0 && (-tanCritAngle > newY/newRad || newRad < 0))
		{
			float len = glm::length(_position);
			newRad = len * cos(-critAngle);
			newY = len * sin(-critAngle);
		}
		_position.x = newRad * _position.x / rad;
		_position.y = newY;
		_position.z = newRad * _position.z / rad;

		_position += _pivot;

		_camera->setTarget(_pivot);
		_camera->setPosition(_position);
		_camera->updateTransform();
	}
}