#ifndef _COMMANDER_H_
#define _COMMANDER_H_

#include "Scene.h"
#include "Mesh.h"
#include "SceneNode.h"
#include <unordered_map>
#include <string>

using namespace std;

/**
	Provides methods to load and save scenes and models.
*/
class Commander
{
public:
	/**
		Inits the Commander object with a scene.
	*/
	Commander(Scene* scene);
	~Commander();

	/**
		If the COLLADA model at `fileName` has been loaded before, returns the mesh. Otherwise loads the model and returns it.
		@param fileName path relative to the models/ folder. Must be a COLLADA *.dae model.
	*/
	Mesh* loadModel(const string& fileName);
	/**
		Creates a new [SceneNode](@ref SceneNode) and appends a new [MeshInstance](@ref MeshInstance) to it. Appends the scene node to the root of the scene and returns it.
		The scene node has an [SRTTransform](@ref SRTTransform) as its local transform.
	*/
	SceneNode* addInstanceToScene(Mesh* mesh);

	/**
		Saves the current scene configuration with its scene nodes and mesh instances into the scenes/ folder and the provided `fileName`.
	*/
	void saveScene(const string& fileName);

	/**
		Clears the current scene and loads a previously saved scene from file `fileName` in the scenes/ folder.
	*/
	void loadScene(const string& fileName);

	/**
		Returns the file name that is associated with `mesh`. If none is, returns an empty string.
	*/
	const string& getMeshName(Mesh* mesh);

private:
	Scene* _scene;
	unordered_map<string, Mesh*> _meshes;
	unordered_map<Mesh*, string> _meshNames;

	static const string EMPTY_STRING;
};

#endif