#ifndef _FRAMEBUFFER_H_
#define _FRAMEBUFFER_H_

#include "include_opengl.h"

/**
	Encapsulates information needed to build a GL framebuffer.
	@see Framebuffer
*/
class FramebufferDescriptor
{
public:
	FramebufferDescriptor(int width, int height);

	int width, height;
	int depthFormat, depthInternalFormat;
	int numColorAttachments;
	int* colorFormat;
	int* colorInternalFormat;

	GLint textureWrap;

	GLint depthMinFilter, depthMagFilter;
	GLint depthType;

	GLint colorMinFilter, colorMagFilter;
	GLenum colorType;
};

/**
	Encapsulated state of a GL framebuffer.
*/
class Framebuffer
{
public:
	Framebuffer();
	/**
		If initialized, deallocate all CPU and GPU memory used by this object.
	*/
	~Framebuffer();

	/**
		Allocates a new GL framebuffer. Number and type of attachments are read from `descriptor`. Call this before calling any other method.
	*/
	bool init(const FramebufferDescriptor& descriptor);

	/**
		Legacy method, use `FramebufferDescriptor` instead
	*/
	bool init(int width, int height, int depthFormat, int depthInternalFormat,
		int numColorAttachments, int* colorFormat, int* colorInternalFormat);

	bool init(int width, int height, int depthFormat, int depthInternalFormat,
		int colorFormat, int colorInternalFormat);

	/**
		Binds this framebuffer to the GL context.
	*/
	void bind() const;
	/**
		Binds the default framebuffer to the GL context.
	*/
	void unbind() const;

	/**
		returns a handle to the depth attachment
	*/
	GLuint getDepthTextureHandle() const;
	/**
		returns a handle to the `i`'s color attachment
	*/
	GLuint getColorTextureHandle(int i=0) const;
	/**
		returns a handle to this framebuffer
	*/
	GLuint getHandle() const;
	/**
		sets the GL viewport to fit this framebuffer
	*/
	void setViewport() const;

	int getWidth() const;
	int getHeight() const;

private:
	GLuint _framebufferHandle;

	int _depthFormat, _depthInternalFormat;
	GLuint _depthTextureHandle;

	int _numColorAttachments;
	int *_colorFormat, *_colorInternalFormat;
	GLuint *_colorTextureHandle;
	GLenum* _colorAttachments;

	int _width, _height;
	bool _initted;
};


#endif