#include "GLFWGameWindow.h"
#include "include_opengl.h"

#include <GL/glfw.h>
#include <iostream>

GLFWGameWindow::GLFWGameWindow(int width, int height, const std::string& title, bool fullscreen, int major, int minor)
	: _width(width), _height(height), _title(title), _fullscreen(fullscreen), _invFPS(0.016666),
	_running(false), _glMajor(major), _glMinor(minor), _printFPS(false), _printFPSInterval(1.5),
	_maxFrameLength(0.1), _minFrameLength(1./60.)
{
}

GLFWGameWindow::~GLFWGameWindow()
{
}

void GLFWGameWindow::setGLVersion(int major, int minor)
{
	if (major >= 1 && minor >= 1)
	{
		_glMajor = major;
		_glMinor = minor;
	}
}

bool GLFWGameWindow::run()
{
	glfwInit();

	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, _glMajor);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, _glMinor);
	glfwOpenWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
	glfwOpenWindowHint(GLFW_WINDOW_NO_RESIZE, GL_TRUE);
	
	if (glfwOpenWindow(_width,_height,0,0,0,0,24,8,_fullscreen?GLFW_FULLSCREEN:GLFW_WINDOW) != GL_TRUE)
	{
		std::cout << "ERROR: failed to init window" << std::endl;
		glfwTerminate();
		return false;
	}
	int ret = flextInit();
	glfwSetWindowTitle(_title.c_str());

	init();

	double startTime, endTime, interval = 0.016666;
	double lastPrint = 0.0;
	_running = true;
	while(_running)
	{
		glfwSetTime(.0);
		startTime = glfwGetTime();

		update(interval);

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		draw();

		// this also polls events, important to call window opened test before doing any more GL
		glfwSwapBuffers();

		endTime = glfwGetTime();
		interval = endTime - startTime;
		_invFPS = 0.98*_invFPS + 0.02*interval;
		if (interval > _maxFrameLength) interval = _maxFrameLength;
		lastPrint += interval;
		if (lastPrint > _printFPSInterval)
		{
			lastPrint = .0;
			if (_printFPS)
			{
				std::cout << getAverageFPS() << " fps" << std::endl;
			}
		}

		_running = _running && !glfwGetKey( GLFW_KEY_ESC );
		
		_running = _running && glfwGetWindowParam(GLFW_OPENED);

		
	}
	destroy();
	glfwTerminate();
	return true;
}

void GLFWGameWindow::setPrintFPS(bool print)
{
	_printFPS = print;
}

void GLFWGameWindow::stop()
{
	_running = false;
}

double GLFWGameWindow::getAverageFPS() const
{
	return 1.0 / _invFPS;
}

int GLFWGameWindow::getWidth() const
{
	return _width;
}

int GLFWGameWindow::getHeight() const
{
	return _height;
}