#ifndef _IDCOUNTER_H_
#define _IDCOUNTER_H_

#include "UserAction.h"
#include <vector>

/**
	Interface for objects that are rendered to the screen and can be selected by the user.
	Each Identifiable has a unique ID that is rendered to an ID buffer to the pixels that the object
	covers on screen.
*/
class Identifiable
{
public:
	Identifiable();

	/**
		returns the unique ID of this object
	*/
	unsigned int getID();
	/**
		returns the objects associated with `id`
	*/
	static Identifiable* byID(unsigned int id);

	/**
		Can be implemented by a subclass. Called every frame by the application iff the mouse cursor rests on the object.
	*/
	virtual void hover();
	
	/**
		Can be implemented by a subcalled. Called by the application if the user clicks the object
		@return `NO_CHANGE` iff the sub class ignores the click event. Otherwise the return value tells the application that
			a certain action has been started and that the object requests to keep the focus.
	*/
	virtual UserAction press();

private:
	unsigned int _id;

	static unsigned int _idCounter;
	static std::vector<Identifiable*> _byID;
};


#endif