#ifndef _INPUTHANDLER_H_
#define _INPUTHANDLER_H_

#include <gl/glfw.h>

/**
	Handler for keyboard and mouse events
*/
class InputHandler
{
public:
	InputHandler();

	/**
		Returns true iff `button` is held down.
	*/
	bool isMouseDown(int button);
	/**
		Returns true iff `button` was pressed down since the last call to `update()`.
	*/
	bool isMousePressed(int button);
	/**
		Returns true iff `button` was released since the last call to `update()`.
	*/
	bool isMouseReleased(int button);

	/**
		Returns true iff `key` is held down.
	*/
	bool isKeyDown(int key);
	/**
		Returns true iff `key` was pressed down since the last call to `update()`.
	*/
	bool isKeyPressed(int key);
	/**
		Returns true iff `key` was released since the last call to `update()`.
	*/
	bool isKeyReleased(int key);

	/**
		Called each frame by the application.
	*/
	void update();

private:
	bool _buttonDown[GLFW_MOUSE_BUTTON_LAST];
	bool _buttonPressed[GLFW_MOUSE_BUTTON_LAST];
	bool _buttonReleased[GLFW_MOUSE_BUTTON_LAST];

	bool _keyDown[256];
	bool _keyPressed[256];
	bool _keyReleased[256];
};




#endif