#include "MeshInstance.h"

#include "ColorMaterial.h"
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_inverse.hpp>
#include <iostream>

MeshInstance::MeshInstance(Mesh* mesh)
	: _mesh(mesh)
{
	_material = new ColorMaterial(glm::vec3(1.f,1.f,1.f));
	_materialType = COLOR_MATERIAL;
}

MeshInstance::~MeshInstance()
{
	delete _material;
}

void MeshInstance::setTransform(const glm::mat4& transform)
{
	_transform = transform;
}

const glm::mat4& MeshInstance::getTransform()
{
	return _transform;
}

Mesh* MeshInstance::getMesh()
{
	return _mesh;
}

void MeshInstance::render(Shader* shader)
{
	glUniformMatrix4fv(shader->getUniformLocation("modelView"), 1, GL_FALSE, glm::value_ptr(_transform));
	glUniformMatrix3fv(shader->getUniformLocation("normalMatrix"), 1, GL_FALSE, glm::value_ptr(glm::mat3(_transform)));
	glUniform1ui(shader->getUniformLocation("colorID"), getID());
	_material->setUniforms(shader);
	_mesh->bindVAO();
	_mesh->draw();
	_mesh->unbindVAO();
}

void MeshInstance::select()
{
	if (_materialType == COLOR_MATERIAL)
	{
		((ColorMaterial*)_material)->setColor(glm::vec3(1.4f,1.4f,1.4f));
	}
}

void MeshInstance::deselect()
{
	if (_materialType == COLOR_MATERIAL)
	{
		((ColorMaterial*)_material)->setColor(glm::vec3(1.0f,1.0f,1.0f));
	}
}



const Material* MeshInstance::_defaultMaterial = new ColorMaterial(glm::vec3(1.f,1.f,1.f));
