#ifndef _MESHINSTANCE_H_
#define _MESHINSTANCE_H_

#include "Mesh.h"
#include "Material.h"
#include "Transformable.h"
#include "Shader.h"
#include "SceneNode.h"
#include "Renderable.h"
#include "Identifiable.h"
#include <vector>
#include <glm/glm.hpp>

using namespace std;

enum MaterialType {
	COLOR_MATERIAL
};

/**
	Instance of a mesh in the scene. Can be part of the [scene graph](@ref SceneNode) and thus transformed freely. The user can manipulate the transform via a `Gizmo`.
	The object can be selected in order to create adaptive cutaways in the scene, see `Renderable`.
*/
class MeshInstance : public Transformable, public Renderable, public Identifiable
{
public:
	MeshInstance(Mesh* mesh);
	virtual ~MeshInstance();

	virtual void setTransform(const glm::mat4& transform);
	virtual const glm::mat4& getTransform();
	Mesh* getMesh();

	virtual void render(Shader* shader);

	virtual void select();
	virtual void deselect();

private:
	Mesh* _mesh;
	Material* _material;
	glm::mat4 _transform;

	static const Material* _defaultMaterial;

	MaterialType _materialType;
};



#endif