#ifndef _MESHLOADER_H_
#define _MESHLOADER_H_

#include "Mesh.h"
#include <string>
#include <assimp/scene.h>

/**
	Provides options on where the origin of a mesh should be placed.
*/
enum Origin
{
	Min,
	Center,
	Max,
	Unchanged
};

/**
	Encapsulates options for loading a mesh from a COLLADA file.
	@see MeshLoader
*/
struct MeshLoadDescription
{
	MeshLoadDescription();

	bool loadNormals;
	bool loadTexCoords;
	bool loadColors;
	bool loadTextures;
	bool moveOrigin;
	Origin originX, originY, originZ;
};

/**
	Loads meshes from a COLLADA file
*/
class MeshLoader
{
public:
	/**
		legacy, use `MeshLoadDescription` instead.
	*/
	static void loadCollada(const std::string& path, Mesh* mesh,
		bool loadNormals=false, bool loadTexCoords=false, bool loadColors=false, bool loadTextures=false);

	/**
		Loads a mesh from a COLLADA file.
		The entire scene graph in the file is flattened to provide a single mesh. Vertex positions are always loaded, optionally normals, texture coordinates and vertex colors are loaded too.
		If vertex colors are present in the file, they are loaded, otherwise the diffuse color of materials in the file is used. If neither is present, all vertex colors are set to white.
		If specified in `descr`, the [origin](@ref Origin) of the mesh is moved.
		@param path path of the model file relative to the models/ folder
		@param [out] mesh stores the loaded mesh
		@param descr Provides options for the loading operation.
	*/
	static void loadCollada(const std::string& path, Mesh* mesh, const MeshLoadDescription& descr);

private:
	static void printMaterial(const aiMaterial& material);
};


#endif