#include "MeshUtils.h"
#include <map>
#include <vector>
#include <iostream>

using namespace std;

///////////////////////////////////////////////////////////////////////
// Adapted from http://prideout.net/blog/?p=54  ///////////////////////
///////////////////////////////////////////////////////////////////////

void MeshUtils::calculateTrianglesAdjacency(int numFaces, const GLuint* srcIndices, GLuint* destIndices,
	int numSrcVertices, const GLfloat* srcVertices, int& numDestVertices,  GLfloat** destVertices)
{
	HalfEdge* allEdges = new HalfEdge[numFaces * 3];

	map<unsigned long long, HalfEdge*>* adjMap = new map<unsigned long long, HalfEdge*>();

	GLuint* copySrcIndices = new GLuint[numFaces * 3];
	memcpy(copySrcIndices, srcIndices, sizeof(GLuint) * numFaces * 3);
	
	GLuint* srcIt = copySrcIndices;
	HalfEdge* edge = allEdges;

	vector<GLfloat> newVertices;
	GLuint newVertexIndex = (GLuint) numSrcVertices;

	for (int i=0; i < numFaces; i++)
	{
		unsigned long long a = *srcIt++;
		unsigned long long b = *srcIt++;
		unsigned long long c = *srcIt++;

		unsigned long long key1 = c | (a << 32);
		unsigned long long key2 = a | (b << 32);
		unsigned long long key3 = b | (c << 32);

		const unsigned int count = adjMap->count(key1) + adjMap->count(key2) + adjMap->count(key3);

		if (count != 0u)
		{
			map<unsigned long long, HalfEdge*>::iterator e1 = adjMap->find(key1);
			map<unsigned long long, HalfEdge*>::iterator e2 = adjMap->find(key2);
			map<unsigned long long, HalfEdge*>::iterator e3 = adjMap->find(key3);

			bool renewA = (e1 != adjMap->end()) || (e1 == adjMap->end() && e2 != adjMap->end() && e3 == adjMap->end());
			bool renewB = (e3 != adjMap->end() && !(e1 != adjMap->end() && e2 == adjMap->end()));

			if (renewA)
			{
				GLuint oldVertexIndex = *(srcIt-3);
				newVertices.push_back(srcVertices[oldVertexIndex*3]);
				newVertices.push_back(srcVertices[oldVertexIndex*3+1]);
				newVertices.push_back(srcVertices[oldVertexIndex*3+2]);
				a = newVertexIndex++;
				*(srcIt-3) = (GLuint)a;
				

				key1 = c | (a << 32);
				key2 = a | (b << 32);
			}

			if (renewB)
			{
				GLuint oldVertexIndex = *(srcIt-2);
				newVertices.push_back(srcVertices[oldVertexIndex*3]);
				newVertices.push_back(srcVertices[oldVertexIndex*3+1]);
				newVertices.push_back(srcVertices[oldVertexIndex*3+2]);
				b = newVertexIndex++;
				*(srcIt-2) = (GLuint)b;
				

				key2 = a | (b << 32);
				key3 = b | (c << 32);
			}
		}

		(*adjMap)[key1] = edge;
		edge->index = (GLuint) a;
		edge->next = 1 + edge;
		++edge;

		(*adjMap)[key2] = edge;
		edge->index = (GLuint) b;
		edge->next = 1 + edge;
		++edge;

		(*adjMap)[key3] = edge;
		edge->index = (GLuint) c;
		edge->next = edge - 2;
		++edge;
	}

	int boundaries = 0;

	numDestVertices = newVertexIndex;
	int numNewVertices = newVertexIndex - (GLuint)numSrcVertices;

	*destVertices = new GLfloat[numDestVertices * 3];
	memcpy(*destVertices, srcVertices, sizeof(GLfloat) * 3 * numSrcVertices);
	memcpy((*destVertices)+3*numSrcVertices, newVertices.data(), sizeof(GLfloat) * 3 * numNewVertices);
	
	HalfEdge* currentEdge;
	map<unsigned long long, HalfEdge*>::iterator it;
	for (it = adjMap->begin(); it != adjMap->end(); it++)
	{
		currentEdge = it->second;
		unsigned long long index = it->first;
		unsigned long long twinIndex = ((index & 0xffffffff) << 32 | (index >> 32));
		
		if (adjMap->count(twinIndex) != 0)
		{
			HalfEdge* twinEdge = (*adjMap)[twinIndex];
			twinEdge->twin = currentEdge;
			currentEdge->twin = twinEdge;
		}
		else
		{
			boundaries++;
			currentEdge->twin = nullptr;
		}
	}

	if (boundaries == 0)
	{
		GLuint* destIndex = destIndices;
		edge = allEdges;
		for (int index=0; index < numFaces; ++index, destIndex+=6, edge+=3)
		{
			destIndex[0] = edge[2].index;
			destIndex[1] = edge[0].twin->next->index;
			destIndex[2] = edge[0].index;
			destIndex[3] = edge[1].twin->next->index;
			destIndex[4] = edge[1].index;
			destIndex[5] = edge[2].twin->next->index;
		}
	}
	else
	{
		std::cout << "WARNING: mesh not two-manifold" << std::endl;
		
		GLuint* destIndex = destIndices;
		edge = allEdges;
		for (int index=0; index < numFaces; ++index, destIndex+=6, edge+=3)
		{
			destIndex[0] = edge[2].index;
			destIndex[1] = (edge[0].twin != nullptr) ? (edge[0].twin->next->index) : destIndex[0];
			destIndex[2] = edge[0].index;
			destIndex[3] = (edge[1].twin != nullptr) ? (edge[1].twin->next->index) : destIndex[2];
			destIndex[4] = edge[1].index;
			destIndex[5] = (edge[2].twin != nullptr) ? (edge[2].twin->next->index) : destIndex[4];
		}
	}
	
	delete[] allEdges;
	delete[] copySrcIndices;

	cout << "Boundaries: " << boundaries << endl;
}