#ifndef _MESHUTILS_H_
#define _MESHUTILS_H_

#include "include_opengl.h"

/**
	Stores half an edge with
	 - index: index of the vertex at the end of the half-edge
	 - twin: the other half of the same edge
	 - next: the next half-edge on the same triangle
 */
class HalfEdge
{
public:
	GLuint index;
	HalfEdge* twin;
	HalfEdge* next;
};

class MeshUtils
{
public:
	/**
	 destIndices must be double the size of srcIndices
	 fills destIndices with the orginal indices + adjacency information
	 @param [in] numFaces number of triangles in the mesh
	 @param [in] srcIndices triangles without adjacency information
	 @param [out] destIndices triangles with adjacency information
	 @param [in] numSrcVertices num of vertices in srcVertices
	 @param [in] srcVertices vertices that srcIndices refers to
	 @param [out] numDestVertices num of vertices in destVertices
	 @param [out] destVertices if there are duplicate edges in srcIndices, new vertices must be generated;
	     will contain a copy of srcVertices plus additionally created vertices; will be allocated
	     by this method
	*/
	static void calculateTrianglesAdjacency(int numFaces, const GLuint* srcIndices, GLuint* destIndices, int numSrcVertices, const GLfloat* srcVertices, int& numDestVertices,  GLfloat** destVertices);
};




#endif _MESHUTILS_H_