#include "Plane.h"

Plane::Plane(const glm::vec3& a, const glm::vec3& b, const glm::vec3& c)
	: _origin(a),
	_normal(glm::normalize(glm::cross(b-a,c-a))),
	_d(-glm::dot(a,_normal))
{
}

Plane::Plane(const glm::vec3& origin, const glm::vec3& normal)
	: _origin(origin),
	_normal(glm::normalize(normal)),
	_d(-glm::dot(origin,normal))
{
}

Plane::~Plane()
{
}

double Plane::signedDistance(const glm::vec3& p) const
{
	return glm::dot(p,_normal) + _d;
}

bool Plane::isFrontFacingTo(const glm::vec3& direction) const {
	return glm::dot(_normal, direction) <= 0.0;
}

bool Plane::lineIntersect(const glm::vec3& linePoint, const glm::vec3& lineDirection, glm::vec3& intersection)
{
	float de = glm::dot(lineDirection, _normal);
	if (abs(de) < 0.00001f)
		return false;
	float en = (float)_d + glm::dot(linePoint, _normal);
	intersection = linePoint - (en/de) * lineDirection;
	return true;
}