#ifndef _PLANE_H_
#define _PLANE_H_

#include <glm/glm.hpp>

/**
	Geometric object of a plane
*/
class Plane
{
public:
	/**
		Creates a plane that contains the three points
	*/
	Plane(const glm::vec3& a, const glm::vec3& b, const glm::vec3& c);
	/**
		Creates a plane that contains `origin` and is orthogonal to `normal`
	*/
	Plane(const glm::vec3& origin, const glm::vec3& normal);
	~Plane();

	/**
		Returns the distance between this plane and p, positive is the normal vector points in the direction of `p`, negative otherwise
	*/
	double signedDistance(const glm::vec3& p) const;
	/**
		Returns true iff `direction` points in the opposite direction of the normal vector.
	*/
	bool isFrontFacingTo(const glm::vec3& direction) const;
	/**
		Returns false iff lineDirection is parallel to this plane, true otherwise.
		Returns the intersection point between this plane and the line described by the origin `linePoint` and the direction `lineDirection`.
		@param [out] intersection intersection point between this plane ant the line
	*/
	bool lineIntersect(const glm::vec3& linePoint, const glm::vec3& lineDirection, glm::vec3& intersection);

private:
	glm::vec3 _origin;
	glm::vec3 _normal;
	double _d;
};

#endif