#ifndef _RENDERABLE_H_
#define _RENDERABLE_H_

#include "Shader.h"

/**
	Interface for 3-dimensional objects that have a visual representation in the scene.

	@see MeshInstance
*/
class Renderable
{
public:
	Renderable();

	/**
		Draws the objects into the scene.
	*/
	virtual void render(Shader* shader)=0;
	/**
		Called by the application if the object is selected by the user
	*/
	virtual void select();
	/**
		Called by the application if the object is deselected by the user
	*/
	virtual void deselect();

	/**
		Sets the `canBeCut` property. If the `canBeCut` property is enabled, the object can be cut out of the seen by selected objects that it occludes.
	*/
	void setCanBeCut(bool c);
	/**
		Returns the `canBeCut` property. If the `canBeCut` property is enabled, the object can be cut out of the seen by selected objects that it occludes.
	*/
	bool canBeCut();

	/**
		Sets the `canCut` property. If the `canCut` property is enabled, the object will exhibit the adaptive cutaway effect once selected.
	*/
	void setCanCut(bool c);
	/**
		Returns the `canCut` property. If the `canCut` property is enabled, the object will exhibit the adaptive cutaway effect once selected.
	*/
	bool canCut();

	/**
		Sets the object-specific angle of the object's cutaway shape.
	*/
	void setLocalAngle(float angle);

	/**
		Returns the object-specific angle of the object's cutaway shape.
	*/
	float getLocalAngle();

private:
	bool _canBeCut;
	bool _canCut;

	float _localAngle;
};


#endif