#ifndef _TERMINAL_H_
#define _TERMINAL_H_

#include "Shader.h"
#include "Mesh.h"
#include "FontRenderer.h"
#include "InputHandler.h"
#include "Commander.h"
#include <deque>
#include <string>

class Terminal;

static Terminal* globalTerminal;

/**
	Draws a command line terminal on the screen. The user can input and execute commands. For a description of the commands, consult the index page.
*/
class Terminal
{
public:
	/**
		Creates a new terminal
		@param scene currently active scene
		@param inputHandler unused
		@param fontRenderer used to draw input text
		@param commander used to load and save scenes
	*/
	Terminal(Scene* scene, InputHandler* inputHandler, FontRenderer* fontRenderer, Commander* commander);
	~Terminal();

	/**
		This application calls this to make the terminal active. This means it will be displayed and register its own key and character callback methods with GLFW.
	*/
	void setActive(bool active);
	bool isActive();

	/**
		Callback method for character events from GLFW.
	*/
	void characterCallback(int character, int action);
	/**
		Callback method for key events from GLFW.
	*/
	void keyCallback(int key, int action);

	/**
		Draws command line and input text
	*/
	void render();
	/**
		Lets the user input text and execute commands
	*/
	void update(double frameLength);

	/**
		Returns true iff the user wants to close the terminal
	*/
	bool requestsInactive();

private:
	Scene* _scene;
	InputHandler* _inputHandler;
	Commander* _commander;

	bool _active;

	Shader _colorShader;

	Mesh* _bgMesh;
	FontRenderer* _fontRenderer;

	Text* _currentText;
	std::deque<Text*> _previousTexts;

	double _blinkTimer;
	std::string _blinkCharacter;
	std::string _currentLine;

	bool _requestInactive;

	void execute(const std::string& input);
};


#endif