#ifndef _TRANSFORM_H_
#define _TRANSFORM_H_

#include <glm/glm.hpp>

/**
	Base class that allows for propagation of hierarchical transformations through the scene graph.
*/
class Transform
{
public:
	/**
		Returns the model transformation which is calculated from the model transformation of this node's parent and the local transformation.
		Also calculated the scale separately.
		@param inhTransform parent transformation
		@param inhScale parent scale
		@param [out] totalScale  total scale of this node
		@return model transformation of this node
	*/
	virtual glm::mat4 propagate(const glm::mat4& inhTransform, float inhScale, float* totalScale) = 0;
	/**
		Called before propagation. Calculates the local transform internally.
	*/
	virtual void update() = 0;

};


#endif _TRANSFORM_H_