#ifndef _VIS2WINDOW_H_
#define _VIS2WINDOW_H_

#include "UserAction.h"
#include "GLFWGameWindow.h"
#include "Shader.h"
#include "Mesh.h"
#include "Framebuffer.h"
#include "Camera.h"
#include "SceneNode.h"
#include "MeshInstance.h"
#include "InputHandler.h"
#include "CameraController.h"
#include "FontRenderer.h"
#include "Terminal.h"
#include "Commander.h"
#include <list>

using namespace std;

class Vis2Window : public GLFWGameWindow
{
public:
	
	Vis2Window();
	~Vis2Window();

	/**
		Initializes shaders, frame buffers and scene
	*/
	virtual void init();
	/**
		Processes events
	*/
	virtual void update(double frameLength);
	/**
		Draws the scene with adaptive cutaways
	*/
	virtual void draw();
	virtual void destroy();

	/**
		Callback method for GLFW character events
	*/
	void characterCallback(int character, int action);
	/**
		Callback method for GLFW key events
	*/
	void keyCallback(int key, int action);

private:
	

	int width, height;

	InputHandler* inputHandler;

	UserAction _currentAction;

	Shader textureScreenShader;
	Shader depthShader;
	Shader silhouetteShader;
	Shader cellShader;
	Shader combineShader;
	Shader cutSurfaceShader;
	Shader idShader;
	Shader gizmoShader;
	Shader colorShader;
	Shader blur1Shader, blur2Shader;
	

	Mesh screenQuad;

	float angle, scale;

	Framebuffer jumpFloodBuffers[2];
	Framebuffer edgeBuffer;
	Framebuffer cellBuffer;
	Framebuffer gauss1Buffer, gauss2Buffer;
	
	Camera* camera;
	CameraController* _cameraController;
	Scene* _scene;
	Commander* _commander;

	glm::vec3 _lightPosition;
	float _farPlane;

	//MeshInstance* _selection;

	FontRenderer* _fontRenderer;
	Terminal* _terminal;
	bool _pressedTab, _pressedDel;
	
	void loadShaders();
	unsigned int pick();
	unsigned int pick(int x, int y);

	void select(MeshInstance* selection);
	void deselect();
	static void setPerspective(Shader* shader, const glm::mat4& perspective, float farPlane);

	Framebuffer* jumpFloodSelection();
	void renderVisibleFrontSurfaces(Framebuffer* distanceTransform, const glm::vec3& viewLightPosition);
	void renderCutSurfaces(Framebuffer* distanceTransform, const glm::vec3& viewLightPosition);
};

static Vis2Window *globalWindow;

#endif